/// <reference types="node" />
import JSZip, { JSZipObject, OutputType } from 'jszip';
import { AutomizerSummary, FileInfo } from '../types/types';
import { IPresentationProps } from '../interfaces/ipresentation-props';
export declare class FileHelper {
    static readFile(location: string): Promise<Buffer>;
    static extractFromArchive(archive: JSZip, file: string, type?: OutputType): Promise<string | number[] | Uint8Array | ArrayBuffer | Blob | Buffer>;
    static removeFromDirectory(archive: JSZip, dir: string, cb: (file: JSZipObject, relativePath: string) => boolean): string[];
    static removeFromArchive(archive: JSZip, file: string): JSZip;
    static extractFileContent(file: Buffer): Promise<JSZip>;
    static getFileExtension(filename: string): string;
    static getFileInfo(filename: string): FileInfo;
    static check(archive: JSZip, file: string): boolean;
    static isArchive(archive: any): void;
    static fileExistsInArchive(archive: JSZip, file: string): boolean;
    /**
     * Copies a file from one archive to another. The new file can have a different name to the origin.
     * @param {JSZip} sourceArchive - Source archive
     * @param {string} sourceFile - file path and name inside source archive
     * @param {JSZip} targetArchive - Target archive
     * @param {string} targetFile - file path and name inside target archive
     * @return {JSZip} targetArchive as an instance of JSZip
     */
    static zipCopy(sourceArchive: JSZip, sourceFile: string, targetArchive: JSZip, targetFile?: string, tmp?: any): Promise<JSZip>;
    static writeOutputFile(location: string, content: Buffer, automizer: IPresentationProps): Promise<AutomizerSummary>;
}
