"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const modify_chart_1 = require("../modify/modify-chart");
const modify_xml_helper_1 = __importDefault(require("./modify-xml-helper"));
class ModifyChartHelper {
}
exports.default = ModifyChartHelper;
/**
 * Set chart data to modify default chart types.
 * See `__tests__/modify-existing-chart.test.js`
 */
ModifyChartHelper.setChartData = (data) => (element, chart, workbook) => {
    const slots = [];
    data.series.forEach((series, s) => {
        slots.push({
            index: s,
            series: series,
            targetCol: s + 1,
            type: 'defaultSeries',
        });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modify();
    // XmlHelper.dump(chart)
    // XmlHelper.dump(workbook.table)
};
/**
 * Set chart data to modify vertical line charts.
 * See `__tests__/modify-chart-vertical-lines.test.js`
 */
ModifyChartHelper.setChartVerticalLines = (data) => (element, chart, workbook) => {
    const slots = [];
    slots.push({
        label: `Y-Values`,
        mapData: (point, category) => category.y,
        targetCol: 1,
    });
    data.series.forEach((series, s) => {
        slots.push({
            index: s,
            series: series,
            targetCol: s + 2,
            type: 'xySeries',
        });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modify();
};
/**
 * Set chart data to modify scatter charts.
 * See `__tests__/modify-chart-scatter.test.js`
 */
ModifyChartHelper.setChartScatter = (data) => (element, chart, workbook) => {
    const slots = [];
    data.series.forEach((series, s) => {
        const colId = s * 2;
        slots.push({
            index: s,
            series: series,
            targetCol: colId + 1,
            type: 'customSeries',
            tag: 'c:xVal',
            mapData: (point) => point.x,
        });
        slots.push({
            label: `${series.label}-Y-Value`,
            index: s,
            series: series,
            targetCol: colId + 2,
            type: 'customSeries',
            tag: 'c:yVal',
            mapData: (point) => point.y,
            isStrRef: false,
        });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modify();
    // XmlHelper.dump(chart)
};
/**
 * Set chart data to modify combo charts.
 * This type is prepared for
 * first series: bar chart (e.g. total)
 * other series: vertical lines
 * See `__tests__/modify-chart-scatter.test.js`
 */
ModifyChartHelper.setChartCombo = (data) => (element, chart, workbook) => {
    const slots = [];
    slots.push({
        index: 0,
        series: data.series[0],
        targetCol: 1,
        type: 'defaultSeries',
    });
    slots.push({
        index: 1,
        label: `Y-Values`,
        mapData: (point, category) => category.y,
        targetCol: 2,
    });
    data.series.forEach((series, s) => {
        if (s > 0)
            slots.push({
                index: s,
                series: series,
                targetCol: s + 2,
                targetYCol: 2,
                type: 'xySeries',
            });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modify();
};
/**
 * Set chart data to modify bubble charts.
 * See `__tests__/modify-chart-bubbles.test.js`
 */
ModifyChartHelper.setChartBubbles = (data) => (element, chart, workbook) => {
    const slots = [];
    data.series.forEach((series, s) => {
        const colId = s * 3;
        slots.push({
            index: s,
            series: series,
            targetCol: colId + 1,
            type: 'customSeries',
            tag: 'c:xVal',
            mapData: (point) => point.x,
        });
        slots.push({
            label: `${series.label}-Y-Value`,
            index: s,
            series: series,
            targetCol: colId + 2,
            type: 'customSeries',
            tag: 'c:yVal',
            mapData: (point) => point.y,
            isStrRef: false,
        });
        slots.push({
            label: `${series.label}-Size`,
            index: s,
            series: series,
            targetCol: colId + 3,
            type: 'customSeries',
            tag: 'c:bubbleSize',
            mapData: (point) => point.size,
            isStrRef: false,
        });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modify();
    // XmlHelper.dump(chart)
};
/**
 * Set chart data to modify extended chart types.
 * See `__tests__/modify-existing-extended-chart.test.js`
 */
ModifyChartHelper.setExtendedChartData = (data) => (element, chart, workbook) => {
    const slots = [];
    data.series.forEach((series, s) => {
        slots.push({
            index: s,
            series: series,
            targetCol: s + 1,
            type: 'extendedSeries',
        });
    });
    new modify_chart_1.ModifyChart(chart, workbook, data, slots).modifyExtended();
    // XmlHelper.dump(chart);
    // XmlHelper.dump(workbook.table)
};
ModifyChartHelper.setAxisRange = (range) => (chart) => {
    const axis = chart.getElementsByTagName('c:valAx')[range.axisIndex || 0];
    if (!axis)
        return;
    ModifyChartHelper.setAxisAttribute(axis, 'c:majorUnit', range.majorUnit);
    ModifyChartHelper.setAxisAttribute(axis, 'c:minorUnit', range.minorUnit);
    ModifyChartHelper.setAxisAttribute(axis, 'c:numFmt', range.formatCode, 'formatCode');
    ModifyChartHelper.setAxisAttribute(axis, 'c:numFmt', range.sourceLinked, 'sourceLinked');
    const scaling = axis.getElementsByTagName('c:scaling')[0];
    ModifyChartHelper.setAxisAttribute(scaling, 'c:min', range.min);
    ModifyChartHelper.setAxisAttribute(scaling, 'c:max', range.max);
};
ModifyChartHelper.setAxisAttribute = (element, tag, value, attribute) => {
    if (value === undefined || !element)
        return;
    const target = element.getElementsByTagName(tag);
    if (target.length > 0) {
        attribute = attribute || 'val';
        if (typeof value === 'boolean') {
            modify_xml_helper_1.default.booleanAttribute(attribute, value)(target[0]);
        }
        else {
            modify_xml_helper_1.default.attribute(attribute, value)(target[0]);
        }
    }
};
//# sourceMappingURL=modify-chart-helper.js.map