import JSZip from 'jszip';
import { DefaultAttribute, HelperElement, OverrideAttribute, RelationshipAttribute, ModifyXmlCallback } from '../types/xml-types';
import { GetRelationshipsCallback, Target } from '../types/types';
export declare class XmlHelper {
    static modifyXmlInArchive(archive: Promise<JSZip>, file: string, callbacks: ModifyXmlCallback[]): Promise<JSZip>;
    static getXmlFromArchive(archive: JSZip, file: string): Promise<XMLDocument>;
    static writeXmlToArchive(archive: JSZip, file: string, xml: XMLDocument): Promise<JSZip>;
    static appendIf(element: HelperElement): Promise<HelperElement | boolean>;
    static append(element: HelperElement): Promise<HelperElement>;
    static removeIf(element: HelperElement): Promise<Element[]>;
    static getNextRelId(rootArchive: JSZip, file: string): Promise<string>;
    static getMaxId(rels: NodeListOf<ChildNode> | HTMLCollectionOf<Element>, attribute: string, increment?: boolean, minId?: number): number;
    static getTargetsFromRelationships(archive: JSZip, path: string, prefix: string | string[]): Promise<Target[]>;
    static pushRelTargets(element: Element, prefix: string, targets: Target[]): void;
    static targetMatchesRelationship(relType: any, subtype: any, target: any, prefix: any): boolean;
    static getTargetsByRelationshipType(archive: JSZip, path: string, type: string): Promise<Target[]>;
    static getRelationships(archive: JSZip, path: string, cb: GetRelationshipsCallback): Promise<Target[]>;
    static getRelationshipItems(archive: JSZip, path: string, tag: string, cb: GetRelationshipsCallback): Promise<Target[]>;
    static findByAttribute(xml: XMLDocument | Document, tagName: string, attributeName: string, attributeValue: string): boolean;
    static replaceAttribute(archive: JSZip, path: string, tagName: string, attributeName: string, attributeValue: string, replaceValue: string): Promise<JSZip>;
    static getTargetByRelId(archive: JSZip, slideNumber: number, element: XMLDocument, type: string): Promise<Target>;
    static findByElementCreationId(archive: JSZip, path: string, creationId: string): Promise<XMLDocument>;
    static findByElementName(archive: JSZip, path: string, name: string): Promise<XMLDocument>;
    static findByName(doc: Document, name: string): XMLDocument;
    static findByCreationId(doc: Document, creationId: string): XMLDocument;
    static findFirstByAttributeValue(nodes: NodeListOf<ChildNode> | HTMLCollectionOf<Element>, attributeName: string, attributeValue: string): Element;
    static findByAttributeValue(nodes: NodeListOf<ChildNode> | HTMLCollectionOf<Element>, attributeName: string, attributeValue: string): Element[];
    static createContentTypeChild(archive: JSZip, attributes: OverrideAttribute | DefaultAttribute): HelperElement;
    static createRelationshipChild(archive: JSZip, targetRelFile: string, attributes: RelationshipAttribute): HelperElement;
    static appendSharedString(sharedStrings: Document, stringValue: string): number;
    static insertAfter(newNode: Node, referenceNode: Element): Node;
    static sliceCollection(collection: HTMLCollectionOf<Element>, length: number, from?: number): void;
    static remove(toRemove: Element): void;
    static sortCollection(collection: HTMLCollectionOf<Element>, order: number[], callback?: ModifyXmlCallback): void;
    static modifyCollection(collection: HTMLCollectionOf<Element>, callback: ModifyXmlCallback): void;
    static dump(element: XMLDocument | Element): void;
}
