"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XmlHelper = void 0;
const xmldom_1 = require("@xmldom/xmldom");
const file_helper_1 = require("./file-helper");
const constants_1 = require("../constants/constants");
const xml_pretty_print_1 = require("./xml-pretty-print");
const lodash_1 = __importDefault(require("lodash"));
const general_helper_1 = require("./general-helper");
const content_tracker_1 = require("./content-tracker");
class XmlHelper {
    static modifyXmlInArchive(archive, file, callbacks) {
        return __awaiter(this, void 0, void 0, function* () {
            const jsZip = yield archive;
            const xml = yield XmlHelper.getXmlFromArchive(jsZip, file);
            let i = 0;
            for (const callback of callbacks) {
                yield callback(xml, i++, jsZip);
            }
            return yield XmlHelper.writeXmlToArchive(yield archive, file, xml);
        });
    }
    static getXmlFromArchive(archive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            const xmlDocument = (yield file_helper_1.FileHelper.extractFromArchive(archive, file));
            const dom = new xmldom_1.DOMParser();
            return dom.parseFromString(xmlDocument);
        });
    }
    static writeXmlToArchive(archive, file, xml) {
        return __awaiter(this, void 0, void 0, function* () {
            const s = new xmldom_1.XMLSerializer();
            const xmlBuffer = s.serializeToString(xml);
            return archive.file(file, xmlBuffer);
        });
    }
    static appendIf(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            return element.clause !== undefined && !element.clause(xml)
                ? false
                : XmlHelper.append(element);
        });
    }
    static append(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            const newElement = xml.createElement(element.tag);
            for (const attribute in element.attributes) {
                const value = element.attributes[attribute];
                const setValue = typeof value === 'function' ? value(xml) : value;
                newElement.setAttribute(attribute, setValue);
            }
            content_tracker_1.contentTracker.trackRelation(element.file, element.attributes);
            if (element.assert) {
                element.assert(xml);
            }
            const parent = element.parent(xml);
            parent.appendChild(newElement);
            yield XmlHelper.writeXmlToArchive(element.archive, element.file, xml);
            return newElement;
        });
    }
    static removeIf(element) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(element.archive, element.file);
            const collection = xml.getElementsByTagName(element.tag);
            const toRemove = [];
            XmlHelper.modifyCollection(collection, (item, index) => {
                if (element.clause(xml, item)) {
                    toRemove.push(item);
                }
            });
            toRemove.forEach((item) => {
                XmlHelper.remove(item);
            });
            yield XmlHelper.writeXmlToArchive(element.archive, element.file, xml);
            return toRemove;
        });
    }
    static getNextRelId(rootArchive, file) {
        return __awaiter(this, void 0, void 0, function* () {
            const presentationRelsXml = yield XmlHelper.getXmlFromArchive(rootArchive, file);
            const increment = (max) => 'rId' + max;
            const relationNodes = presentationRelsXml.documentElement.childNodes;
            const rid = XmlHelper.getMaxId(relationNodes, 'Id', true);
            return increment(rid) + '-created';
        });
    }
    static getMaxId(rels, attribute, increment, minId) {
        let max = 0;
        for (const i in rels) {
            const rel = rels[i];
            if (rel.getAttribute !== undefined) {
                const id = Number(rel
                    .getAttribute(attribute)
                    .replace('rId', '')
                    .replace('-created', ''));
                max = id > max ? id : max;
            }
        }
        switch (typeof increment) {
            case 'boolean':
                ++max;
                break;
        }
        if (max < minId) {
            return minId;
        }
        return max;
    }
    static getTargetsFromRelationships(archive, path, prefix) {
        return __awaiter(this, void 0, void 0, function* () {
            const prefixes = typeof prefix === 'string' ? [prefix] : prefix;
            return XmlHelper.getRelationships(archive, path, (element, targets) => {
                prefixes.forEach((prefix) => {
                    XmlHelper.pushRelTargets(element, prefix, targets);
                });
            });
        });
    }
    static pushRelTargets(element, prefix, targets) {
        const type = element.getAttribute('Type');
        const file = element.getAttribute('Target');
        const rId = element.getAttribute('Id');
        const subtype = lodash_1.default.last(prefix.split('/'));
        const relType = lodash_1.default.last(type.split('/'));
        const filename = lodash_1.default.last(file.split('/'));
        const filenameExt = lodash_1.default.last(filename.split('.'));
        const filenameMatch = filename
            .replace('.' + filenameExt, '')
            .match(/^(.+?)(\d+)*$/);
        const number = filenameMatch && filenameMatch[2] ? Number(filenameMatch[2]) : 0;
        const filenameBase = filenameMatch && filenameMatch[1] ? filenameMatch[1] : filename;
        if (XmlHelper.targetMatchesRelationship(relType, subtype, file, prefix)) {
            targets.push({
                file,
                rId,
                number,
                type,
                subtype,
                prefix,
                filename,
                filenameExt,
                filenameBase,
            });
        }
    }
    static targetMatchesRelationship(relType, subtype, target, prefix) {
        if (relType === 'package')
            return true;
        return relType === subtype && target.indexOf(prefix) === 0;
    }
    static getTargetsByRelationshipType(archive, path, type) {
        return __awaiter(this, void 0, void 0, function* () {
            return XmlHelper.getRelationships(archive, path, (element, rels) => {
                const target = element.getAttribute('Type');
                if (target === type) {
                    rels.push({
                        file: element.getAttribute('Target'),
                        rId: element.getAttribute('Id'),
                    });
                }
            });
        });
    }
    static getRelationships(archive, path, cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getRelationshipItems(archive, path, 'Relationship', cb);
        });
    }
    static getRelationshipItems(archive, path, tag, cb) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(archive, path);
            const relationshipItems = xml.getElementsByTagName(tag);
            const rels = [];
            Object.keys(relationshipItems)
                .map((key) => relationshipItems[key])
                .filter((element) => element.getAttribute !== undefined)
                .forEach((element) => cb(element, rels));
            return rels;
        });
    }
    static findByAttribute(xml, tagName, attributeName, attributeValue) {
        const elements = xml.getElementsByTagName(tagName);
        for (const i in elements) {
            const element = elements[i];
            if (element.getAttribute !== undefined) {
                if (element.getAttribute(attributeName) === attributeValue) {
                    return true;
                }
            }
        }
        return false;
    }
    static replaceAttribute(archive, path, tagName, attributeName, attributeValue, replaceValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const xml = yield XmlHelper.getXmlFromArchive(archive, path);
            const elements = xml.getElementsByTagName(tagName);
            for (const i in elements) {
                const element = elements[i];
                if (element.getAttribute !== undefined &&
                    element.getAttribute(attributeName) === attributeValue) {
                    element.setAttribute(attributeName, replaceValue);
                }
                if (element.getAttribute !== undefined) {
                    content_tracker_1.contentTracker.trackRelation(path, {
                        Id: element.getAttribute('Id'),
                        Target: element.getAttribute('Target'),
                        Type: element.getAttribute('Type'),
                    });
                }
            }
            return XmlHelper.writeXmlToArchive(archive, path, xml);
        });
    }
    static getTargetByRelId(archive, slideNumber, element, type) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = constants_1.TargetByRelIdMap[type];
            const sourceRid = element
                .getElementsByTagName(params.relRootTag)[0]
                .getAttribute(params.relAttribute);
            const relsPath = `ppt/slides/_rels/slide${slideNumber}.xml.rels`;
            const imageRels = yield XmlHelper.getTargetsFromRelationships(archive, relsPath, params.prefix);
            const target = imageRels.find((rel) => rel.rId === sourceRid);
            return target;
        });
    }
    static findByElementCreationId(archive, path, creationId) {
        return __awaiter(this, void 0, void 0, function* () {
            const slideXml = yield XmlHelper.getXmlFromArchive(archive, path);
            return XmlHelper.findByCreationId(slideXml, creationId);
        });
    }
    static findByElementName(archive, path, name) {
        return __awaiter(this, void 0, void 0, function* () {
            const slideXml = yield XmlHelper.getXmlFromArchive(archive, path);
            return XmlHelper.findByName(slideXml, name);
        });
    }
    static findByName(doc, name) {
        const names = doc.getElementsByTagName('p:cNvPr');
        for (const i in names) {
            if (names[i].getAttribute && names[i].getAttribute('name') === name) {
                return names[i].parentNode.parentNode;
            }
        }
        return null;
    }
    static findByCreationId(doc, creationId) {
        const creationIds = doc.getElementsByTagName('a16:creationId');
        for (const i in creationIds) {
            if (creationIds[i].getAttribute &&
                creationIds[i].getAttribute('id') === creationId) {
                return creationIds[i].parentNode.parentNode.parentNode.parentNode
                    .parentNode;
            }
        }
        return null;
    }
    static findFirstByAttributeValue(nodes, attributeName, attributeValue) {
        for (const i in nodes) {
            const node = nodes[i];
            if (node.getAttribute &&
                node.getAttribute(attributeName) === attributeValue) {
                return node;
            }
        }
        return null;
    }
    static findByAttributeValue(nodes, attributeName, attributeValue) {
        const matchingNodes = [];
        for (const i in nodes) {
            const node = nodes[i];
            if (node.getAttribute &&
                node.getAttribute(attributeName) === attributeValue) {
                matchingNodes.push(node);
            }
        }
        return matchingNodes;
    }
    static createContentTypeChild(archive, attributes) {
        return {
            archive,
            file: `[Content_Types].xml`,
            parent: (xml) => xml.getElementsByTagName('Types')[0],
            tag: 'Override',
            attributes,
        };
    }
    static createRelationshipChild(archive, targetRelFile, attributes) {
        content_tracker_1.contentTracker.trackRelation(targetRelFile, attributes);
        return {
            archive,
            file: targetRelFile,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes,
        };
    }
    static appendSharedString(sharedStrings, stringValue) {
        const strings = sharedStrings.getElementsByTagName('sst')[0];
        const newLabel = sharedStrings.createTextNode(stringValue);
        const newText = sharedStrings.createElement('t');
        newText.appendChild(newLabel);
        const newString = sharedStrings.createElement('si');
        newString.appendChild(newText);
        strings.appendChild(newString);
        return strings.getElementsByTagName('si').length - 1;
    }
    static insertAfter(newNode, referenceNode) {
        return referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
    }
    static sliceCollection(collection, length, from) {
        if (from !== undefined) {
            for (let i = from; i < length; i++) {
                XmlHelper.remove(collection[i]);
            }
        }
        else {
            for (let i = collection.length; i > length; i--) {
                XmlHelper.remove(collection[i - 1]);
            }
        }
    }
    static remove(toRemove) {
        toRemove.parentNode.removeChild(toRemove);
    }
    static sortCollection(collection, order, callback) {
        if (collection.length === 0) {
            return;
        }
        const parent = collection[0].parentNode;
        order.forEach((index, i) => {
            if (!collection[index]) {
                (0, general_helper_1.vd)('sortCollection index not found' + index);
                return;
            }
            const item = collection[index];
            if (callback) {
                callback(item, i);
            }
            parent.appendChild(item);
        });
    }
    static modifyCollection(collection, callback) {
        for (let i = 0; i < collection.length; i++) {
            const item = collection[i];
            callback(item, i);
        }
    }
    static dump(element) {
        const s = new xmldom_1.XMLSerializer();
        const xmlBuffer = s.serializeToString(element);
        const p = new xml_pretty_print_1.XmlPrettyPrint(xmlBuffer);
        p.dump();
    }
}
exports.XmlHelper = XmlHelper;
//# sourceMappingURL=xml-helper.js.map